"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var JsonParser = (function () {
    function JsonParser() {
        this.reset('');
        this.keywordMap = {
            'true': 'BOOLEAN',
            'false': 'BOOLEAN',
            'null': 'NULL'
        };
    }
    JsonParser.prototype.determineOtherTokenType = function (lexeme) {
        var tokenType = this.keywordMap[lexeme];
        return tokenType ? tokenType : 'IDENTIFIER';
    };
    JsonParser.prototype.getOtherToken = function () {
        while (this.index < this.code.length) {
            var ch = this.code.charAt(this.index++);
            if (ch === '"' ||
                JsonParser.isNumberStartChar(ch) ||
                JsonParser.isWhitespaceChar(ch) ||
                JsonParser.isBracket(ch) ||
                JsonParser.isSeparatorChar(ch)) {
                this.index--;
                break;
            }
            this.buf += ch;
        }
        return { lexeme: this.buf, type: this.determineOtherTokenType(this.buf) };
    };
    JsonParser.prototype.getStringLexeme = function () {
        var prevEscape = false;
        this.buf += '"'; // Previously read in
        while (this.index < this.code.length) {
            var ch = this.code.charAt(this.index++);
            switch (ch) {
                case '"':
                    this.buf += ch;
                    if (!prevEscape) {
                        return this.buf;
                    }
                    prevEscape = false;
                    break;
                case '\\':
                    prevEscape = true;
                    break;
                default:
                    if (prevEscape) {
                        this.buf += '\\';
                    }
                    this.buf += ch;
                    prevEscape = false;
                    break;
            }
        }
        // Technically invalid - input ended with an unterminated string ending with '\'
        if (prevEscape) {
            this.buf += '\\';
        }
        return this.buf;
    };
    JsonParser.isBracket = function (ch) {
        return ch === '[' ||
            ch === '{' ||
            ch === ']' ||
            ch === '}';
    };
    JsonParser.isNumberChar = function (ch) {
        return (ch >= '0' && ch <= '9') || ch === '-' || ch === '.' || ch === 'e' || ch === 'E';
    };
    JsonParser.isNumberStartChar = function (ch) {
        return (ch >= '0' && ch <= '9') || ch === '-';
    };
    JsonParser.isSeparatorChar = function (ch) {
        return ch === ':' || ch === ',';
    };
    JsonParser.isWhitespaceChar = function (ch) {
        return ch === ' ' ||
            ch === '\r' ||
            ch === '\n' ||
            ch === '\t' ||
            ch === '\f';
    };
    JsonParser.prototype.nextToken = function () {
        if (this.index >= this.code.length) {
            return null;
        }
        this.buf = '';
        var ch = this.code.charAt(this.index++);
        if (JsonParser.isBracket(ch)) {
            return { lexeme: ch, type: 'BRACKET' };
        }
        else if (JsonParser.isWhitespaceChar(ch)) {
            this.buf += ch;
            while (this.index < this.code.length) {
                ch = this.code.charAt(this.index);
                if (JsonParser.isWhitespaceChar(ch)) {
                    this.buf += ch;
                    this.index++;
                }
                else {
                    break;
                }
            }
            return { lexeme: this.buf, type: 'WHITESPACE' };
        }
        else if (ch === '"') {
            return { lexeme: this.getStringLexeme(), type: 'STRING' };
        }
        else if (JsonParser.isNumberStartChar(ch)) {
            this.buf += ch;
            while (this.index < this.code.length) {
                ch = this.code.charAt(this.index);
                if (JsonParser.isNumberChar(ch)) {
                    this.buf += ch;
                    this.index++;
                }
                else {
                    break;
                }
            }
            return { lexeme: this.buf, type: 'NUMBER' };
        }
        else if (JsonParser.isSeparatorChar(ch)) {
            return { lexeme: ch, type: 'SEPARATOR' };
        }
        this.index--;
        return this.getOtherToken();
    };
    JsonParser.prototype.reset = function (code) {
        this.code = code;
        this.state = 'INITIAL';
        this.index = 0;
        this.buf = '';
    };
    return JsonParser;
}());
exports.JsonParser = JsonParser;
//# sourceMappingURL=json-parser.js.map